@@ -19,6 +19,7 @@
  *      Mathias Hasselmann
  */
 #include "tls.h"
+#include "shell.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -30,6 +31,8 @@
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
+#include <uuid/uuid.h>
+
 /**
  * SECTION:tls
  * @short_description: TLS support
@@ -53,13 +56,6 @@ struct _EcpTlsKeyContext
   gint rc;
 };
 
-static gpointer epc_tls_private_key_enter_default (void);
-
-static EpcTlsPrivkeyEnterHook epc_tls_private_key_enter = epc_tls_private_key_enter_default;
-static EpcTlsPrivkeyLeaveHook epc_tls_private_key_leave = NULL;
-
-extern gboolean _epc_debug;
-
 GQuark
 epc_tls_error_quark (void)
 {
@@ -68,12 +64,14 @@ epc_tls_error_quark (void)
 
 static gchar*
 epc_tls_get_filename (const gchar *hostname,
-                      const gchar *category)
+                      const gchar *extension)
 {
   const gchar *progname = g_get_prgname ();
+  gchar *filename = NULL;
+  gchar *basename = NULL;
 
   g_return_val_if_fail (NULL != hostname, NULL);
-  g_return_val_if_fail (NULL != category, NULL);
+  g_return_val_if_fail (NULL != extension, NULL);
 
   if (NULL == progname)
     {
@@ -84,8 +82,13 @@ epc_tls_get_filename (const gchar *hostname,
       progname = "";
     }
 
-  return g_build_filename (g_get_user_config_dir (), progname, 
-                           "libepc", category, hostname, NULL);
+  basename = g_strconcat (hostname, extension, NULL);
+  filename = g_build_filename (g_get_user_config_dir (),
+                               "libepc", progname,
+                               basename, NULL);
+
+  g_free (basename);
+  return filename;
 }
 
 /**
@@ -94,15 +97,15 @@ epc_tls_get_filename (const gchar *hostname,
  *
  * Queries the preferred location for storing private X.509 keys for the server
  * identified by @hostname. This file is located in the current user's XDG
- * configuration folder (#g_get_user_config_dir). The filename also contains 
- * program's name when specified by #g_set_prgname.
+ * configuration folder (g_get_user_config_dir()). The filename also contains
+ * program's name when specified by g_set_prgname().
  *
  * Returns: The preferred private key location for @hostname.
  */
 gchar*
 epc_tls_get_private_key_filename (const gchar *hostname)
 {
-  return epc_tls_get_filename (hostname, "keys");
+  return epc_tls_get_filename (hostname, ".key");
 }
 
 /**
@@ -111,48 +114,15 @@ epc_tls_get_private_key_filename (const gchar *hostname)
  *
  * Queries the preferred location for storing X.509 certificates for the server
  * identified by @hostname. This file is located in the current user's XDG
- * configuration folder (#g_get_user_config_dir). The filename also contains 
- * program's name when specified by #g_set_prgname.
+ * configuration folder (g_get_user_config_dir()). The filename also contains
+ * program's name when specified by g_set_prgname().
  *
  * Returns: The preferred certificate location for @hostname.
  */
 gchar*
 epc_tls_get_certificate_filename (const gchar *hostname)
 {
-  return epc_tls_get_filename (hostname, "certs");
-}
-
-static gpointer
-epc_tls_private_key_enter_default (void)
-{
-  g_print (_("Generating server key. This may take some time. "
-             "Type on the keyboard, move your mouse or "
-             "browse the web to generate some entropy.\n"));
-
-  return NULL;
-}
-
-/**
- * epc_tls_set_private_key_hooks:
- * @enter: the function to call on start
- * @leave: the function to call when finished
- *
- * Installs functions which are called when private keys have to be generated.
- *
- * Generating secure keys needs quite some time, so those functions shall be
- * called to provide  some feedback to your users. Key generation takes place
- * in a separate background thread, whilst the calling thread waits in a
- * GMainLoop. So for instance the GTK+ widget system remains responsible during
- * that phase.
- *
- * See also: #EpcEntropyProgress
- */
-void
-epc_tls_set_private_key_hooks (EpcTlsPrivkeyEnterHook enter,
-                               EpcTlsPrivkeyLeaveHook leave)
-{
-  epc_tls_private_key_enter = enter;
-  epc_tls_private_key_leave = leave;
+  return epc_tls_get_filename (hostname, ".crt");
 }
 
 static gpointer
@@ -171,7 +141,7 @@ epc_tls_private_key_thread (gpointer data)
  * @error: return location for a #GError, or %NULL
  *
  * Creates a self private X.509 key. Generating secure keys needs quite
- * some time. Call #epc_tls_set_private_key_hooks to install hooks providing
+ * some time. Call epc_tls_set_private_key_hooks() to install hooks providing
  * some feedback to your users. Key generation takes place in a separate
  * background thread, whilst the calling thread waits in a GMainLoop. So
  * for instance the GTK+ widget system remains responsible during that
@@ -191,10 +161,11 @@ gnutls_x509_privkey_t
 epc_tls_private_key_new (GError **error)
 {
   EcpTlsKeyContext context = { NULL, NULL, GNUTLS_E_SUCCESS };
-  gpointer private_key_data = NULL;
 
-  if (epc_tls_private_key_enter)
-    private_key_data = epc_tls_private_key_enter ();
+  epc_shell_progress_begin (_("Generating Server Key"),
+                            _("This may take some time. Type on the "
+                              "keyboard, move your mouse, or browse "
+                              "the web to generate some entropy."));
 
   context.rc = gnutls_x509_privkey_init (&context.key);
   epc_tls_check (context.rc);
@@ -212,8 +183,7 @@ epc_tls_private_key_new (GError **error)
   epc_tls_check (context.rc);
 
 out:
-  if (epc_tls_private_key_leave)
-    epc_tls_private_key_leave (private_key_data);
+  epc_shell_progress_end ();
 
   if (GNUTLS_E_SUCCESS != context.rc)
     {
@@ -258,7 +228,7 @@ epc_tls_private_key_load (const gchar *filename,
 
   if (g_file_get_contents (filename, &contents, &buffer.size, error))
     {
-      if (G_UNLIKELY (_epc_debug))
+      if (EPC_DEBUG_LEVEL (1))
         g_debug ("%s: Loading private key `%s'", G_STRLOC, filename);
 
       buffer.data = (guchar*) contents;
@@ -315,7 +285,7 @@ epc_tls_private_key_save (gnutls_x509_privkey_t  key,
   g_return_val_if_fail (NULL != key, FALSE);
   g_return_val_if_fail (NULL != filename, FALSE);
 
-  if (G_UNLIKELY (_epc_debug))
+  if (EPC_DEBUG_LEVEL (1))
     g_debug ("%s: Writing server key `%s'", G_STRLOC, filename);
 
   rc = gnutls_x509_privkey_export (key, GNUTLS_X509_FMT_PEM, NULL, &length);
@@ -330,7 +300,7 @@ epc_tls_private_key_save (gnutls_x509_privkey_t  key,
                    _("Cannot export private key to PEM format: %s"),
                    gnutls_strerror (rc));
 
-      goto out; 
+      goto out;
     }
 
   if (g_mkdir_with_parents (dirname = g_path_get_dirname (filename), 0700))
@@ -419,17 +389,20 @@ epc_tls_certificate_new (const gchar            *hostname,
   gint rc = GNUTLS_E_SUCCESS;
   gnutls_x509_crt_t crt = NULL;
   time_t now = time (NULL);
+  uuid_t serial;
 
   g_return_val_if_fail (NULL != key, NULL);
   g_return_val_if_fail (NULL != hostname, NULL);
 
-  if (G_UNLIKELY (_epc_debug))
+  if (EPC_DEBUG_LEVEL (1))
     g_debug ("%s: Generating self signed server certificate for `%s'", G_STRLOC, hostname);
 
+  uuid_generate_time (serial);
+
   epc_tls_check (rc = gnutls_x509_crt_init (&crt));
   epc_tls_check (rc = gnutls_x509_crt_set_version (crt, 1));
   epc_tls_check (rc = gnutls_x509_crt_set_key (crt, key));
-  epc_tls_check (rc = gnutls_x509_crt_set_serial (crt, "", 1));
+  epc_tls_check (rc = gnutls_x509_crt_set_serial (crt, serial, sizeof serial));
   epc_tls_check (rc = gnutls_x509_crt_set_activation_time (crt, now));
   epc_tls_check (rc = gnutls_x509_crt_set_expiration_time (crt, now + validity));
   epc_tls_check (rc = gnutls_x509_crt_set_subject_alternative_name (crt, GNUTLS_SAN_DNSNAME, hostname));
@@ -481,7 +454,7 @@ epc_tls_certificate_load (const gchar *filename,
 
   if (g_file_get_contents (filename, &contents, &buffer.size, error))
     {
-      if (G_UNLIKELY (_epc_debug))
+      if (EPC_DEBUG_LEVEL (1))
         g_debug ("%s: Loading server certificate `%s'", G_STRLOC, filename);
 
       buffer.data = (guchar*) contents;
@@ -537,7 +510,7 @@ epc_tls_certificate_save (gnutls_x509_crt_t  certificate,
   g_return_val_if_fail (NULL != certificate, FALSE);
   g_return_val_if_fail (NULL != filename, FALSE);
 
-  if (G_UNLIKELY (_epc_debug))
+  if (EPC_DEBUG_LEVEL (1))
     g_debug ("%s: Writing server certificate `%s'", G_STRLOC, filename);
 
   rc = gnutls_x509_crt_export (certificate, GNUTLS_X509_FMT_PEM, NULL, &length);
@@ -552,7 +525,7 @@ epc_tls_certificate_save (gnutls_x509_crt_t  certificate,
                    _("Cannot export server certificate to PEM format: %s"),
                    gnutls_strerror (rc));
 
-      goto out; 
+      goto out;
     }
 
   if (g_mkdir_with_parents (dirname = g_path_get_dirname (filename), 0700))
@@ -588,8 +561,8 @@ out:
  * @error: return location for a #GError, or %NULL
  *
  * Searches or creates X.509 certificate and key for the server identified
- * by @hostname. This function uses #epc_tls_get_certificate_filename and
- * #epc_tls_get_private_key_filename to locate existing certificates and
+ * by @hostname. This function uses epc_tls_get_certificate_filename() and
+ * epc_tls_get_private_key_filename() to locate existing certificates and
  * keys. New certificates and keys are generated, when the files cannot
  * be found, or the existing files contain invalid or expired information.
  *
@@ -653,7 +626,7 @@ epc_tls_get_server_credentials (const gchar  *hostname,
         invalid = FALSE;
 
       if (invalid)
-        { 
+        {
           g_warning ("%s: Discarding invalid server certificate.", G_STRLOC);
           gnutls_x509_crt_deinit (crt);
           crt = NULL;
